from django.shortcuts import render,redirect
from rest_framework.response import Response
from rest_framework import status
from rest_framework.views import APIView
from clmainApp.models import *
from clmainApp.serializers import *
from brmainApp.models import Location
from brmainApp.serializers import LocationSerializer
from django.contrib.auth.decorators import login_required,user_passes_test
from django.utils.decorators import method_decorator
# Create your views here.
from django.apps import apps
from django.forms.models import model_to_dict
from clmainApp.utils import getBranchId
class AddObjectByNameApi(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.PrintReport'), login_url='/main/login/'))
    def post(self,request):
        data = request.data
        if "modelName" in data and "objectName" in data  :
            try : 
                field_name_to_check = "created_by"
                model = apps.get_model(data["modelName"])
                if field_name_to_check in [field.name for field in model._meta.get_fields()]:
                    newObj = model.objects.create(name=data["objectName"],created_by=request.user)
                else : newObj = model.objects.create(name=data["objectName"])
                return Response(model_to_dict(newObj),status=status.HTTP_201_CREATED)
            except: return Response(status=status.HTTP_400_BAD_REQUEST)
        else : return Response(status=status.HTTP_400_BAD_REQUEST)


def test_apis(request):
    return render(request,'Clinic/test_apis.html')


@login_required(login_url='clauthApp:login')
def welcome(request):
    #check if user has permissions to access this page and add any session variables in this function 
    # or add it in login function 
    #user_clinic_image = Clinic.objects.get(id=request.session['clinicId']).welcome_img.url
    #return render(request,'welcome.html',{"user_clinic_image":user_clinic_image})
    clinic = Clinic.objects.all().first()
    if clinic:
        #print("Session")
        request.session['clinicName'] = clinic.name
        request.session['clinicId'] = clinic.id
        request.session['welcomeImg'] = clinic.welcome_img.url
        request.session['iconImg'] = clinic.icon_img.url
        request.session['reportsCode'] = clinic.reports_code
        request.session['SessionPeriod'] = clinic.session_period
        return render(request,'Clinic/clinic_welcome.html')
    else : return redirect('clauthApp:home')



class NationalitiesApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_nationalities'), login_url='/auth/login/'))
    def get(self,request):
        #if not getClinicId(request) : return redirect('/main/login/')
        Nationalities =Nationality.objects.all()
        serializer=NationalitySerializer(Nationalities,many=True)
        return render(request,"Clinic/clmainApp/nationalities.html",{"all_nationalities":serializer.data,'activePage':'Nationalities'})
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.add_nationality'), login_url='/auth/login/'))    
    def post(self,request):
        #if not getClinicId(request) : return redirect('/main/login/')
        serializer= NationalitySerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(status=status.HTTP_201_CREATED)
        else :return Response(serializer.errors,status=status.HTTP_400_BAD_REQUEST)

class NationalityApi(APIView):
    def get_object(self,pk):
        try:
            return Nationality.objects.get(pk=pk)
        except Nationality.DoesNotExist:
            return None    
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_nationalities'), login_url='/auth/login/'))        
    def get(self,request,pk): 
        NationalityObj=self.get_object(pk)
        if NationalityObj :
            serializer= NationalitySerializer(NationalityObj)
            return Response(serializer.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.change_nationality'), login_url='/auth/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        serializer= NationalitySerializer(obj,data=request.data)
        #list_error=""
        if serializer.is_valid():
            serializer.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializer.errors,status=status.HTTP_400_BAD_REQUEST)          

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.delete_nationality'), login_url='/auth/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

class LocationsApi(APIView):
    #user_passes_test(lambda u: any(u.has_perm(perm) for perm in ['ClManagement.view_locations', 'ClManagement.other_permission'])
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_locations'), login_url='/auth/login/'))
    def get(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        #if not getClinicId(request) : return redirect('/main/login/')
        all_locations =Location.objects.filter(branch_id=branchId)
        all_locations =LocationSerializer(all_locations,many=True)
        return render(request,"Branch/clmainApp/locations.html",{"all_locations":all_locations.data,'activePage':'Locations'})
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.add_location'), login_url='/auth/login/'))    
    def post(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        data = request.data
        data._mutable = True
        data.update({"created_by":request.user.id,"branch":branchId})
        #if not getClinicId(request) : return redirect('/main/login/')
        serializer= LocationSerializer(data=data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data,status=status.HTTP_201_CREATED)
        else :return Response(serializer.errors,status=status.HTTP_400_BAD_REQUEST)

class LocationApi(APIView):
    def get_object(self,pk):
        try:
            return Location.objects.get(pk=pk)
        except Location.DoesNotExist:
            return None    
        
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_locations'), login_url='/auth/login/'))        
    def get(self,request,pk): 
        Obj=self.get_object(pk)
        if Obj :
            serializer= LocationSerializer(Obj)
            return Response(serializer.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.change_location'), login_url='/auth/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        request.data._mutable = True
        request.data.update({"created_by":request.user.id})
        serializer= LocationSerializer(obj,data=request.data,partial=True)
        if serializer.is_valid():
            serializer.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializer.errors,status=status.HTTP_400_BAD_REQUEST)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.delete_location'), login_url='/auth/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

class FoundWaysApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_foundways'), login_url='/auth/login/'))
    def get(self,request):
        #if not getClinicId(request) : return redirect('/main/login/')
        all_found_ways =FoundWay.objects.all()
        serializer =FoundWaySerializer(all_found_ways,many=True)
        return render(request,"Clinic/clmainApp/found_ways.html",{"all_found_ways":serializer.data,'activePage':'FoundWays'})
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.add_foundway'), login_url='/auth/login/'))    
    def post(self,request):
        #if not getClinicId(request) : return redirect('/main/login/')
        serializer= FoundWaySerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data,status=status.HTTP_201_CREATED)
        else :return Response(serializer.errors,status=status.HTTP_400_BAD_REQUEST)

class FoundWayApi(APIView):
    def get_object(self,pk):
        try:
            return FoundWay.objects.get(pk=pk)
        except FoundWay.DoesNotExist:
            return None    
        
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_foundways'), login_url='/auth/login/'))        
    def get(self,request,pk): 
        Obj=self.get_object(pk)
        if Obj :
            serializer= FoundWaySerializer(Obj)
            return Response(serializer.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.change_foundway'), login_url='/auth/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        serializer= FoundWaySerializer(obj,data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializer.errors,status=status.HTTP_400_BAD_REQUEST)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.delete_foundway'), login_url='/auth/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

class PatientStatusesApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_pt_statuses'), login_url='/auth/login/'))
    def get(self,request):
        #if not getClinicId(request) : return redirect('/main/login/')
        all_pt_statuses =PatientStatus.objects.all()
        serializer =PatientStatusSerializer(all_pt_statuses,many=True)
        return render(request,"Clinic/clmainApp/pt_statuses.html",{"all_pt_statuses":serializer.data,'activePage':'PatientStatuses'})
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.add_pt_status'), login_url='/auth/login/'))    
    def post(self,request):
        #if not getClinicId(request) : return redirect('/main/login/')
        data = request.data
        data._mutable = True
        data.update({"created_by":request.user.id})
        serializer= PatientStatusSerializer(data=data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data,status=status.HTTP_201_CREATED)
        else :return Response(serializer.errors,status=status.HTTP_400_BAD_REQUEST)

class PatientStatusApi(APIView):
    def get_object(self,pk):
        try:
            return PatientStatus.objects.get(pk=pk)
        except PatientStatus.DoesNotExist:
            return None
        
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_pt_statuses'), login_url='/auth/login/'))        
    def get(self,request,pk): 
        Obj=self.get_object(pk)
        if Obj :
            serializer= PatientStatusSerializer(Obj)
            return Response(serializer.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.change_pt_status'), login_url='/auth/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        serializer= PatientStatusSerializer(obj,data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializer.errors,status=status.HTTP_400_BAD_REQUEST)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.delete_pt_status'), login_url='/auth/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

class ComplaintsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_complaints'), login_url='/auth/login/'))
    def get(self,request):
        #if not getClinicId(request) : return redirect('/main/login/')
        all_complaints =Complaint.objects.all()
        all_complaints =ComplaintSerializer(all_complaints,many=True)
        return render(request,"Clinic/clmainApp/complaints.html",{"all_complaints":all_complaints.data,'activePage':'Complaints'})
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.add_complaint'), login_url='/auth/login/'))    
    def post(self,request):
        #if not getClinicId(request) : return redirect('/main/login/')
        serializer= ComplaintSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data,status=status.HTTP_201_CREATED)
        else :return Response(serializer.errors,status=status.HTTP_400_BAD_REQUEST)

class ComplaintApi(APIView):
    def get_object(self,pk):
        try:
            return Complaint.objects.get(pk=pk)
        except Complaint.DoesNotExist:
            return None    
        
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_complaints'), login_url='/auth/login/'))        
    def get(self,request,pk): 
        Obj=self.get_object(pk)
        if Obj :
            serializer= ComplaintSerializer(Obj)
            return Response(serializer.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.change_complaint'), login_url='/auth/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        serializer= ComplaintSerializer(obj,data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializer.errors,status=status.HTTP_400_BAD_REQUEST)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.delete_complaint'), login_url='/auth/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

from brvisitApp.models import PatientVisit
from clpresApp.models import PresDrugItem,PresRadiologyItem,PresTestItem
from clpresApp.serializers import PresDrugItemSerializer,PresRadiologyItemSerializer,PresTestItemSerializer

class PrintReportApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.PrintReport'), login_url='/main/login/'))
    def post(self,request,master_pk):
        #master_pk
        #اي حاجه اساسية انا عايز اطبعها 
        #زى مثلا لو روشتة هبعت 
        # patient visit ID 
        # هروح اجيب منه الادوية والتحاليل والاشعة وتوقيتاتها 
        #واجيب منه برضه ملف/كود  المريض واسمه وميعاد الازيارة القادمة 
        #request.data._mutable = True
        report_name=request.data.get("report_name")
        context={}
        #ممكن روشتة - ممكن فاتورة - ممكن تقرير 
        if report_name == "prescription_pv" :#master_pk is patient visit used to collect all data
            template_name = "Clinic/reports/{}/prescription_pv.html".format(request.session['reports_code'])
            visit_id=master_pk
            patientVisitObj = PatientVisit.objects.get(pk=visit_id)
            context.update({"pV":patientVisitObj})
            presDrugItems=PresDrugItem.objects.filter(patient_visit=patientVisitObj)
            presDrugItems_serialized=PresDrugItemSerializer(presDrugItems,many=True)
            context.update({"presDrugItems":presDrugItems_serialized.data})
            presRadiologyItems=PresRadiologyItem.objects.filter(patient_visit=patientVisitObj)
            presRadiologyItems_serialized=PresRadiologyItemSerializer(presRadiologyItems,many=True)
            context.update({"presRadiologyItems":presRadiologyItems_serialized.data})
            presTestItems=PresTestItem.objects.filter(patient_visit=patientVisitObj)
            presTestItems_serialized=PresTestItemSerializer(presTestItems,many=True)
            context.update({"presTestItems":presTestItems_serialized.data})
            return render(request,template_name,context)
        

class CancelReasonsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_cancelreasons'), login_url='/auth/login/'))
    def get(self,request):
        #if not getClinicId(request) : return redirect('/main/login/')
        all_reasons =CancelReason.objects.all()
        all_reasons =CancelReasonSerializer(all_reasons,many=True)
        return render(request,"Clinic/clmainApp/cancel_reasons.html",{"all_reasons":all_reasons.data,
                                                                       "reason_category_choices":REASON_CATEGORY_CHOICES,
                                                                       'activePage':'CancelReasons'})
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.add_cancelreason'), login_url='/auth/login/'))    
    def post(self,request):
        #if not getClinicId(request) : return redirect('/main/login/')
        serializer= CancelReasonSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data,status=status.HTTP_201_CREATED)
        else :return Response(serializer.errors,status=status.HTTP_400_BAD_REQUEST)


class CancelReasonApi(APIView):
    def get_object(self,pk):
        try:
            return CancelReason.objects.get(pk=pk)
        except CancelReason.DoesNotExist:
            return None    
        
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_cancelreasons'), login_url='/auth/login/'))        
    def get(self,request,pk): 
        Obj=self.get_object(pk)
        if Obj :
            serializer= CancelReasonSerializer(Obj)
            return Response(serializer.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.change_cancelreason'), login_url='/auth/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        serializer= CancelReasonSerializer(obj,data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializer.errors,status=status.HTTP_400_BAD_REQUEST)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.delete_cancelreason'), login_url='/auth/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)



class DailyAlertMessagesApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_daily_messages'), login_url='/main/login/'))
    def get(self,request):
        #if not getClinicId(request) : return redirect('/main/login/')
        alert_messages =AlertMessage.objects.filter(is_daily=True)#هى هى نفس كل حاجه لكن اليومى مفيش تاريخ 
        serializer=AlertMessageSerializer(alert_messages,many=True)#is_daily send false to POST AlertMessagesApi 
        return render(request,"Clinic/clmainApp/daily_alert_messages.html",{"all_messages":serializer.data,'activePage':'DailyAlertMessages'})
    
class AlertMessagesApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_alert_messages'), login_url='/main/login/'))
    def get(self,request):
        #if not getClinicId(request) : return redirect('/main/login/')
        alert_messages =AlertMessage.objects.filter(is_daily=False)
        serializer=AlertMessageSerializer(alert_messages,many=True)
        return render(request,"Clinic/clmainApp/alert_messages.html",{"all_messages":serializer.data,'activePage':'AlertMessages'})
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.add_alert_message'), login_url='/main/login/'))    
    def post(self,request):
        #if not getClinicId(request) : return redirect('/main/login/') 
        data = request.data
        data._mutable = True
        data["is_daily"]=True if data["is_daily"] == "True" else False
        data["is_date"]=True if data["is_date"] == "True" else False
        serializedObj= AlertMessageSerializer(data=data)
        if serializedObj.is_valid():
            #if data["is_active"] == "True":AlertMessage.objects.all().update(is_active=False)
            serializedObj.save()
            return Response(status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

class AlertMessageApi(APIView):
    def get_object(self,pk):
        try:
            return AlertMessage.objects.get(pk=pk)
        except AlertMessage.DoesNotExist:
            return None    
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_alert_messages'), login_url='/main/login/'))        
    def get(self,request,pk): 
        obj=self.get_object(pk)
        if obj :
            serializeobj= AlertMessageSerializer(obj)
            return Response(serializeobj.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.change_alert_message'), login_url='/main/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        data=request.data
        serializedObj= AlertMessageSerializer(obj,data=data)
        #list_error=""
        if serializedObj.is_valid():
            #if data["is_active"] == "True":AlertMessage.objects.all().update(is_active=False)
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)          

    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.delete_alert_message'), login_url='/main/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)